import {
  createItemTree,
  collapseTree,
  deactivateTree,
  toggleExpandedItemWithId,
  toggleExpandedItemWithLink,
  activateItemWithId,
  activateItemWithLink,
  animateItemWithId,
} from './side-nav-helpers';

describe('createItemTree', () => {
  const jsonItems = [
    { id: 1,
      label: 'Item 1',
      items: [
        { id: 11,
          label: 'Item 1.1',
          link: '/item11',
        },
        { id: 12,
          label: 'Item 1.2',
          link: '/item12',
        },
      ],
    },
    { id: 2,
      label: 'Item 2',
      items: [
        { id: 21,
          label: 'Item 2.1',
          items: [
            { id: 211,
              label: 'Item 2.1.1',
              link: '/item211',
            },
            { id: 212,
              label: 'Item 2.1.2',
              link: '/item212',
            },
          ],
        },
        { id: 22,
          label: 'Item 2.2',
          link: '/item22',
        },
      ],
    },
    { id: 3,
      label: 'Item 3',
      link: '/item3',
    },
  ];

  const expected = [
    { id: 1,
      label: 'Item 1',
      active: false,
      expanded: false,
      level: 0,
      animationState: 0,
      items: [
        { id: 11,
          label: 'Item 1.1',
          active: false,
          expanded: false,
          link: '/item11',
          level: 1,
          animationState: 0,
        },
        { id: 12,
          label: 'Item 1.2',
          active: false,
          expanded: false,
          link: '/item12',
          level: 1,
          animationState: 0,
        },
      ],
    },
    { id: 2,
      label: 'Item 2',
      active: false,
      expanded: false,
      level: 0,
      animationState: 0,
      items: [
        { id: 21,
          label: 'Item 2.1',
          active: false,
          expanded: false,
          level: 1,
          animationState: 0,
          items: [
            { id: 211,
              label: 'Item 2.1.1',
              active: false,
              expanded: false,
              link: '/item211',
              level: 2,
              animationState: 0,
            },
            { id: 212,
              label: 'Item 2.1.2',
              active: false,
              expanded: false,
              link: '/item212',
              level: 2,
              animationState: 0,
            },
          ],
        },
        { id: 22,
          label: 'Item 2.2',
          active: false,
          expanded: false,
          link: '/item22',
          level: 1,
          animationState: 0,
        },
      ],
    },
    { id: 3,
      label: 'Item 3',
      active: false,
      expanded: false,
      link: '/item3',
      level: 0,
      animationState: 0,
    },
  ];

  test('should not mutate the input', () => {
    const result = createItemTree(jsonItems);

    expect(result).not.toBe(jsonItems);
  });

  test('should create the item tree', () => {
    const result = createItemTree(jsonItems);

    expect(result).toEqual(expected);
  });
});

describe('collapseTree', () => {
  const jsonItems = [
    { id: 1,
      label: 'Item 1',
      active: true,
      expanded: true,
      items: [
        { id: 11,
          label: 'Item 1.1',
          active: true,
          expanded: true,
          link: '/item11',
        },
        { id: 12,
          label: 'Item 1.2',
          active: true,
          expanded: true,
          link: '/item12',
        },
      ],
    },
    { id: 2,
      label: 'Item 2',
      active: true,
      expanded: true,
      items: [
        { id: 21,
          label: 'Item 2.1',
          active: true,
          expanded: true,
          items: [
            { id: 211,
              label: 'Item 2.1.1',
              active: true,
              expanded: true,
              link: '/item211',
            },
            { id: 212,
              label: 'Item 2.1.2',
              active: true,
              expanded: true,
              link: '/item212',
            },
          ],
        },
        { id: 22,
          label: 'Item 2.2',
          active: true,
          expanded: true,
          link: '/item22',
        },
      ],
    },
    { id: 3,
      label: 'Item 3',
      active: true,
      expanded: true,
      link: '/item3',
    },
  ];

  const expected = [
    { id: 1,
      label: 'Item 1',
      active: true,
      expanded: false,
      items: [
        { id: 11,
          label: 'Item 1.1',
          active: true,
          expanded: false,
          link: '/item11',
        },
        { id: 12,
          label: 'Item 1.2',
          active: true,
          expanded: false,
          link: '/item12',
        },
      ],
    },
    { id: 2,
      label: 'Item 2',
      active: true,
      expanded: false,
      items: [
        { id: 21,
          label: 'Item 2.1',
          active: true,
          expanded: false,
          items: [
            { id: 211,
              label: 'Item 2.1.1',
              active: true,
              expanded: false,
              link: '/item211',
            },
            { id: 212,
              label: 'Item 2.1.2',
              active: true,
              expanded: false,
              link: '/item212',
            },
          ],
        },
        { id: 22,
          label: 'Item 2.2',
          active: true,
          expanded: false,
          link: '/item22',
        },
      ],
    },
    { id: 3,
      label: 'Item 3',
      active: true,
      expanded: false,
      link: '/item3',
    },
  ];

  test('should not mutate the input', () => {
    const result = collapseTree(jsonItems);

    expect(result).not.toBe(jsonItems);
  });

  test('should collapse all the open subtrees', () => {
    const result = collapseTree(jsonItems);

    expect(result).toEqual(expected);
  });
});

describe('toggleExpandedItemWithId', () => {
  const input1 = [
    { id: 1,
      label: 'Item 1',
      expanded: false,
      active: false,
      items: [
        { id: 11,
          label: 'Item 1.1',
          expanded: false,
          active: false,
          link: '/item11',
        },
        { id: 12,
          label: 'Item 1.2',
          expanded: false,
          active: false,
          link: '/item12',
        },
      ],
    },
    { id: 2,
      label: 'Item 2',
      expanded: false,
      active: false,
      items: [
        { id: 21,
          label: 'Item 2.1',
          expanded: false,
          active: false,
          items: [
            { id: 211,
              label: 'Item 2.1.1',
              expanded: false,
              active: false,
              link: '/item211',
            },
            { id: 212,
              label: 'Item 2.1.2',
              expanded: false,
              active: false,
              link: '/item212',
            },
          ],
        },
        { id: 22,
          label: 'Item 2.2',
          expanded: false,
          active: false,
          link: '/item22',
        },
      ],
    },
    { id: 3,
      label: 'Item 3',
      expanded: false,
      active: false,
      link: '/item3',
    },
  ];

  const expected1 = [
    { id: 1,
      label: 'Item 1',
      expanded: false,
      active: false,
      items: [
        { id: 11,
          label: 'Item 1.1',
          expanded: false,
          active: false,
          link: '/item11',
        },
        { id: 12,
          label: 'Item 1.2',
          expanded: false,
          active: false,
          link: '/item12',
        },
      ],
    },
    { id: 2,
      label: 'Item 2',
      expanded: true,
      active: false,
      items: [
        { id: 21,
          label: 'Item 2.1',
          expanded: false,
          active: false,
          items: [
            { id: 211,
              label: 'Item 2.1.1',
              expanded: false,
              active: false,
              link: '/item211',
            },
            { id: 212,
              label: 'Item 2.1.2',
              expanded: false,
              active: false,
              link: '/item212',
            },
          ],
        },
        { id: 22,
          label: 'Item 2.2',
          expanded: false,
          active: false,
          link: '/item22',
        },
      ],
    },
    { id: 3,
      label: 'Item 3',
      expanded: false,
      active: false,
      link: '/item3',
    },
  ];

  const input2 = [
    { id: 1,
      label: 'Item 1',
      expanded: true,
      active: false,
      items: [
        { id: 11,
          label: 'Item 1.1',
          expanded: false,
          active: false,
          link: '/item11',
        },
        { id: 12,
          label: 'Item 1.2',
          expanded: false,
          active: false,
          link: '/item12',
        },
      ],
    },
    { id: 2,
      label: 'Item 2',
      expanded: false,
      active: false,
      items: [
        { id: 21,
          label: 'Item 2.1',
          expanded: false,
          active: false,
          items: [
            { id: 211,
              label: 'Item 2.1.1',
              expanded: false,
              active: false,
              link: '/item211',
            },
            { id: 212,
              label: 'Item 2.1.2',
              expanded: false,
              active: false,
              link: '/item212',
            },
          ],
        },
        { id: 22,
          label: 'Item 2.2',
          expanded: false,
          active: false,
          link: '/item22',
        },
      ],
    },
    { id: 3,
      label: 'Item 3',
      expanded: false,
      active: false,
      link: '/item3',
    },
  ];

  const expected2 = [
    { id: 1,
      label: 'Item 1',
      expanded: false,
      active: false,
      items: [
        { id: 11,
          label: 'Item 1.1',
          expanded: false,
          active: false,
          link: '/item11',
        },
        { id: 12,
          label: 'Item 1.2',
          expanded: false,
          active: false,
          link: '/item12',
        },
      ],
    },
    { id: 2,
      label: 'Item 2',
      expanded: true,
      active: false,
      items: [
        { id: 21,
          label: 'Item 2.1',
          expanded: false,
          active: false,
          items: [
            { id: 211,
              label: 'Item 2.1.1',
              expanded: false,
              active: false,
              link: '/item211',
            },
            { id: 212,
              label: 'Item 2.1.2',
              expanded: false,
              active: false,
              link: '/item212',
            },
          ],
        },
        { id: 22,
          label: 'Item 2.2',
          expanded: false,
          active: false,
          link: '/item22',
        },
      ],
    },
    { id: 3,
      label: 'Item 3',
      expanded: false,
      active: false,
      link: '/item3',
    },
  ];

  const input3 = [
    { id: 1,
      label: 'Item 1',
      expanded: true,
      active: false,
      items: [
        { id: 11,
          label: 'Item 1.1',
          expanded: false,
          active: false,
          link: '/item11',
        },
        { id: 12,
          label: 'Item 1.2',
          expanded: false,
          active: false,
          link: '/item12',
        },
      ],
    },
    { id: 2,
      label: 'Item 2',
      expanded: true,
      active: false,
      items: [
        { id: 21,
          label: 'Item 2.1',
          expanded: true,
          active: false,
          link: '/item21',
          items: [
            { id: 211,
              label: 'Item 2.1.1',
              expanded: false,
              active: false,
              link: '/item211',
            },
            { id: 212,
              label: 'Item 2.1.2',
              expanded: false,
              active: false,
              link: '/item212',
            },
          ],
        },
        { id: 22,
          label: 'Item 2.2',
          expanded: false,
          active: false,
          link: '/item22',
        },
      ],
    },
    { id: 3,
      label: 'Item 3',
      expanded: false,
      active: false,
      link: '/item3',
    },
  ];

  const expected3 = [
    { id: 1,
      label: 'Item 1',
      expanded: false,
      active: false,
      items: [
        { id: 11,
          label: 'Item 1.1',
          expanded: false,
          active: false,
          link: '/item11',
        },
        { id: 12,
          label: 'Item 1.2',
          expanded: false,
          active: false,
          link: '/item12',
        },
      ],
    },
    { id: 2,
      label: 'Item 2',
      expanded: true,
      active: false,
      items: [
        { id: 21,
          label: 'Item 2.1',
          expanded: false,
          active: false,
          link: '/item21',
          items: [
            { id: 211,
              label: 'Item 2.1.1',
              expanded: false,
              active: false,
              link: '/item211',
            },
            { id: 212,
              label: 'Item 2.1.2',
              expanded: false,
              active: false,
              link: '/item212',
            },
          ],
        },
        { id: 22,
          label: 'Item 2.2',
          expanded: false,
          active: false,
          link: '/item22',
        },
      ],
    },
    { id: 3,
      label: 'Item 3',
      expanded: false,
      active: false,
      link: '/item3',
    },
  ];

  const input4 = [
    { id: 1,
      label: 'Item 1',
      expanded: false,
      active: false,
      items: [
        { id: 11,
          label: 'Item 1.1',
          expanded: false,
          active: false,
          link: '/item11',
        },
        { id: 12,
          label: 'Item 1.2',
          expanded: false,
          active: false,
          link: '/item12',
        },
      ],
    },
    { id: 2,
      label: 'Item 2',
      expanded: false,
      active: false,
      items: [
        { id: 21,
          label: 'Item 2.1',
          expanded: false,
          active: false,
          link: '/item21',
          items: [
            { id: 211,
              label: 'Item 2.1.1',
              expanded: false,
              active: false,
              link: '/item211',
            },
            { id: 212,
              label: 'Item 2.1.2',
              expanded: false,
              active: false,
              link: '/item212',
            },
          ],
        },
        { id: 22,
          label: 'Item 2.2',
          expanded: false,
          active: false,
          link: '/item22',
        },
      ],
    },
    { id: 3,
      label: 'Item 3',
      expanded: false,
      active: false,
      link: '/item3',
    },
  ];

  const expected4 = [
    { id: 1,
      label: 'Item 1',
      expanded: false,
      active: false,
      items: [
        { id: 11,
          label: 'Item 1.1',
          expanded: false,
          active: false,
          link: '/item11',
        },
        { id: 12,
          label: 'Item 1.2',
          expanded: false,
          active: false,
          link: '/item12',
        },
      ],
    },
    { id: 2,
      label: 'Item 2',
      expanded: true,
      active: false,
      items: [
        { id: 21,
          label: 'Item 2.1',
          expanded: true,
          active: false,
          link: '/item21',
          items: [
            { id: 211,
              label: 'Item 2.1.1',
              expanded: true,
              active: false,
              link: '/item211',
            },
            { id: 212,
              label: 'Item 2.1.2',
              expanded: false,
              active: false,
              link: '/item212',
            },
          ],
        },
        { id: 22,
          label: 'Item 2.2',
          expanded: false,
          active: false,
          link: '/item22',
        },
      ],
    },
    { id: 3,
      label: 'Item 3',
      expanded: false,
      active: false,
      link: '/item3',
    },
  ];

  test('should not mutate the input', () => {
    const result = toggleExpandedItemWithId(2, input1);

    expect(result).not.toBe(input1);
  });

  test('should open subtree of item 2', () => {
    const result = toggleExpandedItemWithId(2, input1);

    expect(result).toEqual(expected1);
  });

  test('should close subtree of item 2 again', () => {
    const result = toggleExpandedItemWithId(2, expected1);

    expect(result).toEqual(input1);
  });

  test('should open subtree of item 2 while item 1 is expanded -> causing item 1 to collapse', () => {
    const result = toggleExpandedItemWithId(2, input2);

    expect(result).toEqual(expected2);
  });

  test('should close subtree of item 21 while item 2 is staying expanded', () => {
    const result = toggleExpandedItemWithId(21, input3);

    expect(result).toEqual(expected3);
  });

  test('should close subtree of item 21 while item 2 is staying expanded while using the link to search', () => {
    const result = toggleExpandedItemWithLink('/item21', input3);

    expect(result).toEqual(expected3);
  });

  test('should expand the parent element if a child is expanded', () => {
    const result = toggleExpandedItemWithId(211, input4);

    expect(result).toEqual(expected4);
  });
});

describe('activateItem', () => {
  const input1 = [
    { id: 1,
      label: 'Item 1',
      expanded: false,
      active: false,
      items: [
        { id: 11,
          label: 'Item 1.1',
          expanded: false,
          active: false,
          link: '/item11',
        },
        { id: 12,
          label: 'Item 1.2',
          expanded: false,
          active: false,
          link: '/item12',
        },
      ],
    },
    { id: 2,
      label: 'Item 2',
      expanded: true,
      active: false,
      items: [
        { id: 21,
          label: 'Item 2.1',
          expanded: false,
          active: false,
          items: [
            { id: 211,
              label: 'Item 2.1.1',
              expanded: false,
              active: false,
              link: '/item211',
            },
            { id: 212,
              label: 'Item 2.1.2',
              expanded: false,
              active: false,
              link: '/item212',
            },
          ],
        },
        { id: 22,
          label: 'Item 2.2',
          expanded: false,
          active: false,
          link: '/item22',
        },
      ],
    },
    { id: 3,
      label: 'Item 3',
      expanded: false,
      active: false,
      link: '/item3',
    },
  ];

  const expected1 = [
    { id: 1,
      label: 'Item 1',
      expanded: false,
      active: false,
      items: [
        { id: 11,
          label: 'Item 1.1',
          expanded: false,
          active: false,
          link: '/item11',
        },
        { id: 12,
          label: 'Item 1.2',
          expanded: false,
          active: false,
          link: '/item12',
        },
      ],
    },
    { id: 2,
      label: 'Item 2',
      expanded: true,
      active: true,
      items: [
        { id: 21,
          label: 'Item 2.1',
          expanded: false,
          active: true,
          items: [
            { id: 211,
              label: 'Item 2.1.1',
              expanded: false,
              active: false,
              link: '/item211',
            },
            { id: 212,
              label: 'Item 2.1.2',
              expanded: false,
              active: true,
              link: '/item212',
            },
          ],
        },
        { id: 22,
          label: 'Item 2.2',
          expanded: false,
          active: false,
          link: '/item22',
        },
      ],
    },
    { id: 3,
      label: 'Item 3',
      expanded: false,
      active: false,
      link: '/item3',
    },
  ];

  const input2 = [
    { id: 1,
      label: 'Item 1',
      expanded: false,
      active: false,
      items: [
        { id: 11,
          label: 'Item 1.1',
          expanded: false,
          active: false,
          link: '/item11',
        },
        { id: 12,
          label: 'Item 1.2',
          expanded: false,
          active: false,
          link: '/item12',
        },
      ],
    },
    { id: 2,
      label: 'Item 2',
      expanded: true,
      active: true,
      items: [
        { id: 21,
          label: 'Item 2.1',
          expanded: false,
          active: true,
          items: [
            { id: 211,
              label: 'Item 2.1.1',
              expanded: false,
              active: true,
              link: '/item211',
            },
            { id: 212,
              label: 'Item 2.1.2',
              expanded: false,
              active: false,
              link: '/item212',
            },
          ],
        },
        { id: 22,
          label: 'Item 2.2',
          expanded: false,
          active: false,
          link: '/item22',
        },
      ],
    },
    { id: 3,
      label: 'Item 3',
      expanded: false,
      active: false,
      link: '/item3',
    },
  ];

  const expected2 = [
    { id: 1,
      label: 'Item 1',
      expanded: false,
      active: false,
      items: [
        { id: 11,
          label: 'Item 1.1',
          expanded: false,
          active: false,
          link: '/item11',
        },
        { id: 12,
          label: 'Item 1.2',
          expanded: false,
          active: false,
          link: '/item12',
        },
      ],
    },
    { id: 2,
      label: 'Item 2',
      expanded: true,
      active: true,
      items: [
        { id: 21,
          label: 'Item 2.1',
          expanded: false,
          active: true,
          items: [
            { id: 211,
              label: 'Item 2.1.1',
              expanded: false,
              active: false,
              link: '/item211',
            },
            { id: 212,
              label: 'Item 2.1.2',
              expanded: false,
              active: false,
              link: '/item212',
            },
          ],
        },
        { id: 22,
          label: 'Item 2.2',
          expanded: false,
          active: false,
          link: '/item22',
        },
      ],
    },
    { id: 3,
      label: 'Item 3',
      expanded: false,
      active: false,
      link: '/item3',
    },
  ];

  test('should not mutate the input', () => {
    const result = activateItemWithLink('/item212', input1);

    expect(result).not.toBe(expected1);
  });

  test('should activate item 2.1.2 and all its parents when searching by link', () => {
    const result = activateItemWithLink('/item212', input1);

    expect(result).toEqual(expected1);
  });

  test('should activate item 2.1.2 and all its parents when searching by id', () => {
    const result = activateItemWithId(212, input1);

    expect(result).toEqual(expected1);
  });

  test('should activate item 2.1 and all its parents while deactivating its children when searching by id', () => {
    const result = activateItemWithId(21, input2);

    expect(result).toEqual(expected2);
  });
});

describe('deactivateTree', () => {
  const input1 = [
    { id: 1,
      label: 'Item 1',
      expanded: false,
      active: false,
      items: [
        { id: 11,
          label: 'Item 1.1',
          expanded: false,
          active: false,
          link: '/item11',
        },
        { id: 12,
          label: 'Item 1.2',
          expanded: false,
          active: false,
          link: '/item12',
        },
      ],
    },
    { id: 2,
      label: 'Item 2',
      expanded: true,
      active: true,
      items: [
        { id: 21,
          label: 'Item 2.1',
          expanded: true,
          active: true,
          items: [
            { id: 211,
              label: 'Item 2.1.1',
              expanded: false,
              active: false,
              link: '/item211',
            },
            { id: 212,
              label: 'Item 2.1.2',
              expanded: false,
              active: true,
              link: '/item212',
            },
          ],
        },
        { id: 22,
          label: 'Item 2.2',
          expanded: false,
          active: false,
          link: '/item22',
        },
      ],
    },
    { id: 3,
      label: 'Item 3',
      expanded: false,
      active: false,
      link: '/item3',
    },
  ];

  const expected1 = [
    { id: 1,
      label: 'Item 1',
      expanded: false,
      active: false,
      items: [
        { id: 11,
          label: 'Item 1.1',
          expanded: false,
          active: false,
          link: '/item11',
        },
        { id: 12,
          label: 'Item 1.2',
          expanded: false,
          active: false,
          link: '/item12',
        },
      ],
    },
    { id: 2,
      label: 'Item 2',
      expanded: true,
      active: false,
      items: [
        { id: 21,
          label: 'Item 2.1',
          expanded: true,
          active: false,
          items: [
            { id: 211,
              label: 'Item 2.1.1',
              expanded: false,
              active: false,
              link: '/item211',
            },
            { id: 212,
              label: 'Item 2.1.2',
              expanded: false,
              active: false,
              link: '/item212',
            },
          ],
        },
        { id: 22,
          label: 'Item 2.2',
          expanded: false,
          active: false,
          link: '/item22',
        },
      ],
    },
    { id: 3,
      label: 'Item 3',
      expanded: false,
      active: false,
      link: '/item3',
    },
  ];

  test('should not mutate the input', () => {
    const result = deactivateTree(input1);

    expect(result).not.toBe(expected1);
  });

  test('should deactivate all items', () => {
    const result = deactivateTree(input1);

    expect(result).toEqual(expected1);
  });
});

describe('animateItemWithId', () => {
  const input1 = [
    { id: 1,
      label: 'Item 1',
      expanded: false,
      active: false,
      animationTime: 250,
      animationState: 0,
      items: [
        { id: 11,
          label: 'Item 1.1',
          expanded: false,
          active: false,
          link: '/item11',
          animationState: 0,
        },
        { id: 12,
          label: 'Item 1.2',
          expanded: false,
          active: false,
          link: '/item12',
          animationState: 0,
        },
      ],
    },
    { id: 2,
      label: 'Item 2',
      expanded: true,
      active: false,
      animationState: 0,
      items: [
        { id: 21,
          label: 'Item 2.1',
          expanded: false,
          active: false,
          animationState: 0,
          items: [
            { id: 211,
              label: 'Item 2.1.1',
              expanded: false,
              active: false,
              link: '/item211',
              animationState: 0,
            },
            { id: 212,
              label: 'Item 2.1.2',
              expanded: false,
              active: false,
              link: '/item212',
              animationState: 0,
            },
          ],
        },
        { id: 22,
          label: 'Item 2.2',
          expanded: false,
          active: false,
          link: '/item22',
          animationState: 0,
        },
      ],
    },
    { id: 3,
      label: 'Item 3',
      expanded: false,
      active: false,
      link: '/item3',
      animationState: 0,
    },
  ];

  const expected1 = [
    { id: 1,
      label: 'Item 1',
      expanded: false,
      active: false,
      animationTime: 250,
      animationState: 1,
      items: [
        { id: 11,
          label: 'Item 1.1',
          expanded: false,
          active: false,
          link: '/item11',
          animationState: 0,
        },
        { id: 12,
          label: 'Item 1.2',
          expanded: false,
          active: false,
          link: '/item12',
          animationState: 0,
        },
      ],
    },
    { id: 2,
      label: 'Item 2',
      expanded: true,
      active: false,
      animationState: 0,
      items: [
        { id: 21,
          label: 'Item 2.1',
          expanded: false,
          active: false,
          animationState: 0,
          items: [
            { id: 211,
              label: 'Item 2.1.1',
              expanded: false,
              active: false,
              link: '/item211',
              animationState: 0,
            },
            { id: 212,
              label: 'Item 2.1.2',
              expanded: false,
              active: false,
              link: '/item212',
              animationState: 0,
            },
          ],
        },
        { id: 22,
          label: 'Item 2.2',
          expanded: false,
          active: false,
          link: '/item22',
          animationState: 0,
        },
      ],
    },
    { id: 3,
      label: 'Item 3',
      expanded: false,
      active: false,
      link: '/item3',
      animationState: 0,
    },
  ];

  const input2 = [
    { id: 1,
      label: 'Item 1',
      expanded: false,
      active: false,
      animationTime: 250,
      animationState: 1,
      items: [
        { id: 11,
          label: 'Item 1.1',
          expanded: false,
          active: false,
          link: '/item11',
          animationState: 0,
        },
        { id: 12,
          label: 'Item 1.2',
          expanded: false,
          active: false,
          link: '/item12',
          animationState: 0,
        },
      ],
    },
    { id: 2,
      label: 'Item 2',
      expanded: true,
      active: false,
      animationState: 0,
      items: [
        { id: 21,
          label: 'Item 2.1',
          expanded: false,
          active: false,
          animationState: 0,
          items: [
            { id: 211,
              label: 'Item 2.1.1',
              expanded: false,
              active: false,
              link: '/item211',
              animationState: 0,
            },
            { id: 212,
              label: 'Item 2.1.2',
              expanded: false,
              active: false,
              link: '/item212',
              animationState: 0,
            },
          ],
        },
        { id: 22,
          label: 'Item 2.2',
          expanded: false,
          active: false,
          link: '/item22',
          animationState: 0,
        },
      ],
    },
    { id: 3,
      label: 'Item 3',
      expanded: false,
      active: false,
      link: '/item3',
      animationState: 0,
    },
  ];

  const expected2 = [
    { id: 1,
      label: 'Item 1',
      expanded: false,
      active: false,
      animationTime: 250,
      animationState: 2,
      items: [
        { id: 11,
          label: 'Item 1.1',
          expanded: false,
          active: false,
          link: '/item11',
          animationState: 0,
        },
        { id: 12,
          label: 'Item 1.2',
          expanded: false,
          active: false,
          link: '/item12',
          animationState: 0,
        },
      ],
    },
    { id: 2,
      label: 'Item 2',
      expanded: true,
      active: false,
      animationState: 0,
      items: [
        { id: 21,
          label: 'Item 2.1',
          expanded: false,
          active: false,
          animationState: 0,
          items: [
            { id: 211,
              label: 'Item 2.1.1',
              expanded: false,
              active: false,
              link: '/item211',
              animationState: 0,
            },
            { id: 212,
              label: 'Item 2.1.2',
              expanded: false,
              active: false,
              link: '/item212',
              animationState: 0,
            },
          ],
        },
        { id: 22,
          label: 'Item 2.2',
          expanded: false,
          active: false,
          link: '/item22',
          animationState: 0,
        },
      ],
    },
    { id: 3,
      label: 'Item 3',
      expanded: false,
      active: false,
      link: '/item3',
      animationState: 0,
    },
  ];

  test('should not mutate the input', () => {
    const result = animateItemWithId(1, input1);

    expect(result).not.toBe(expected1);
  });

  test('should set animationState of item 1 to "1"', () => {
    const result = animateItemWithId(1, input1);

    expect(result).toEqual(expected1);
  });

  test('should set animationState of item 1 to "2"', () => {
    const result = animateItemWithId(1, input2);

    expect(result).toEqual(expected2);
  });

  test('should set animationState of item 1 back to "0"', () => {
    const result = animateItemWithId(1, expected2);

    expect(result).toEqual(input1);
  });
});
